//
// This file is part of the Marble Virtual Globe.
//
// This program is free software licensed under the GNU LGPL. You can
// find a copy of this license in LICENSE.txt in the top directory of
// the source code.
//
// Copyright 2012      Dennis Nienhüser <earthwings@gentoo.org>
//
#include "mypaintlayer.h"

#include <marble/MarbleWidget.h>
#include <marble/MarbleMap.h>
#include <marble/MarbleModel.h>
#include <marble/GeoPainter.h>
#include <marble/GeoDataLineString.h>
#include <marble/LayerInterface.h>
#include <marble/ViewportParams.h>

#include <QTime>
#include <QKeyEvent>

using namespace Marble;

MyPaintLayer::MyPaintLayer(MarbleWidget* widget) : m_widget(widget), m_index(0), m_opacity(180), m_brushEnabled(false), m_center(new GeoDataCoordinates())
{
    // nothing to do
}

QStringList MyPaintLayer::renderPosition() const
{
    // We will paint in exactly one of the following layers.
    // The current one can be changed by pressing the '+' key
    QStringList layers = QStringList() << "SURFACE" << "HOVERS_ABOVE_SURFACE";
    layers << "ORBIT" << "USER_TOOLS" << "STARS";

    int index = m_index % layers.size();
    return QStringList() << layers.at(index);
}

bool MyPaintLayer::eventFilter(QObject *obj, QEvent *event)
{
    Q_UNUSED(obj)

    // Adjust the current layer when '+' is pressed
    if (event->type() == QEvent::KeyPress)
    {
        QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
        if (keyEvent->key() == Qt::Key_Plus) {
            ++m_index;
            // Have window title reflect the current paint layer
            m_widget->setWindowTitle(renderPosition().first());
            return true;
        } else if ( keyEvent->key() == Qt::Key_O )
        {
            ++m_opacity;
            return true;
        }
    }

    return false;
}
    
void MyPaintLayer::setCenter(const GeoDataCoordinates & center)
{
   *m_center = center;
}

GeoDataCoordinates MyPaintLayer::approximate(const GeoDataCoordinates &base, qreal angle, qreal dist)
{
    // This is just a rough estimation that ignores projections.
    // It only works for short distances. Don't use in real code.
    GeoDataCoordinates::Unit deg = GeoDataCoordinates::Degree;
    return GeoDataCoordinates ( base.longitude(deg) + 1.5 * dist * sin(angle),
				base.latitude(deg) + dist * cos(angle), 0.0, deg);
}
    

QSizeF mapFromGeo(const Marble::ViewportParams * viewport, const Marble::GeoDataCoordinates & center, const qreal & range) 
{
   qreal xc, yc, xp, yp;
   
   viewport->screenCoordinates(
         center.longitude(GeoDataCoordinates::Radian),
         center.latitude(GeoDataCoordinates::Radian),
         xc, 
         yc);

   // width
   //GeoDataCoordinates right(center.moveByBearing(M_PI_2, range));//range/EARTH_RADIUS));
   //GeoDataCoordinates right = MyPaintLayer::approximate(center, M_PI_2, range);
   GeoDataCoordinates right = MyPaintLayer::approximate(center, M_PI * 15 / 30.0, 1.0);
   viewport->screenCoordinates(
         right.longitude(GeoDataCoordinates::Radian),
         right.latitude(GeoDataCoordinates::Radian),
         xp, 
         yp);
   qreal width = std::abs(xp-xc)*2.0;

   // height
   //GeoDataCoordinates top(center.moveByBearing(0.0, range));//range/EARTH_RADIUS));
   //GeoDataCoordinates top = MyPaintLayer::approximate(center, 0.0, range);
   GeoDataCoordinates top = MyPaintLayer::approximate(center, M_PI * 0 / 30.0, 1.0);
   viewport->screenCoordinates(
         top.longitude(GeoDataCoordinates::Radian),
         top.latitude(GeoDataCoordinates::Radian),
         xp, 
         yp);
   qreal height = std::abs(yp-yc)*2.0;

   return QSizeF(width, height);
}


bool MyPaintLayer::render( GeoPainter *painter, ViewportParams *viewport,
    const QString& renderPos, GeoSceneLayer * layer )
{
    Q_UNUSED(viewport)
    Q_UNUSED(renderPos)
    Q_UNUSED(layer)

    QTime now = QTime::currentTime();
    qDebug() << __FILE__<<":" << __LINE__<<":"<< __FUNCTION__<<"(): " 
       << " renderPosition:" << renderPosition().first();

    painter->setRenderHint(QPainter::Antialiasing, true);

    // draw background circle
    if ( m_brushEnabled ) 
    {
       painter->save();
       painter->setBrush(QBrush(QColor::fromRgb(0, 0, 0, m_opacity)));
       QSizeF size = mapFromGeo(viewport, *m_center, 1.0);
       painter->drawEllipse(*m_center, size.width(), size.height());
       painter->restore();
    }
     
    // Large circle built by 60 small circles
    painter->setPen( QPen(QBrush(QColor::fromRgb(255,255,255,200)), 3.0, Qt::SolidLine, Qt::RoundCap ) );

    for (int i=0; i<60; ++i)
    {
        painter->drawEllipse(approximate(*m_center, M_PI * i / 30.0, 1.0), 5, 5);
   }

    // hour, minute, second hand
    painter->drawPolyline(GeoDataLineString() << *m_center << approximate(*m_center, M_PI * now.minute() / 30.0, 0.75));
    painter->drawPolyline(GeoDataLineString() << *m_center << approximate(*m_center, M_PI * now.hour() / 6.0, 0.5));
    painter->setPen(QPen(QBrush(Qt::red), 4.0, Qt::SolidLine, Qt::RoundCap ));
    painter->drawPolyline(GeoDataLineString() << *m_center << approximate(*m_center, M_PI * now.second() / 30.0, 1.0));

    return true;
}
